/*
    (C) SergeySoft 2023 z-print@bk.ru
    Re-spread book from Scan - делит отсканированные шитые на скобу СПУСКИ А3, в книгу а4 с правильным порядком страниц
    Re-face A3 to A4 - делит РАЗВОРОТЫ на отдельные страницы
    Restore book last cover reoder - восстанавливает книгу, первым разворотом идет спуск обложка "последняя-первая", потом развороты "2-3", "4-5" и т. д.
    30/07/2023 - добавлена обработка (назначение/переназначение trimbox и cropbox, без этого были проблемы в сторонних программах)
    06/09/2023 - исправлена мелкая ошибка с прогресс-баром, глючило под DC32, под DC64 работало без проблем, сейчас работает везде
*/    


//  функции и тексты в меню, настройка меню
procs = [
    ["Re-face A3 to A4", "splitFaces()"],
    ["Re-spread book from Scan", "splitAndReoder()"],
    ["Restore book last cover reoder", "restoreBookAndCover()"],
];

if (app.viewerVersion < 10) {
    for (proc of procs) 
        app.addMenuItem({ cName: proc[0], cUser: proc[0], cParent: "Tools", cExec: proc[1], cEnable: "event.rc = (event.target != null);"});
} else {
    for (proc of procs) 
        app.addToolButton({ cName: proc[0], cLabel: proc[0], cExec: proc[1], cEnable: "event.rc = (event.target != null);"});   
};

const SPREADSIZE = 4; //  количество страниц в комплекте А3


//  режет РАЗВОРОТЫ на левые и правые страницы, 
splitFaces = app.trustedFunction(function() {
    splitInHalf("_split.pdf");

});

//  Восстанавливает страницы из спуска книги на ознакомление. 
//  подразумевается, что первым разворотом идет обложка "последняя-первая", а потом развороты 2-3, 4-5 и т. д.
restoreBookAndCover = app.trustedFunction(function() {
    splitInHalf("_pages.pdf");
    //  переставим последнюю обложку в конец файла
    this.movePage(0, this.numPages-1);
});


//  режет СПУСКИ тетрадей любого объема пополам и переупорядочивает страницы в верном порядке
splitAndReoder = app.trustedFunction(function() {
    //  спуск состоит из лица и оборота (n раз), если нечетное количество листов, то отсканировано с ошибкой
    //  вываливаемся с ошибкой 
    if (this.numPages % 2) {
        app.alert ("Error on Scan\nIt must contain EVEN page count");
        return;
    }
    //  порежем копию исходного файла пополам и сделаем ее активной
    splitInHalf("_reodered.pdf");

    try {

        var spreadCount = this.numPages / SPREADSIZE; //  количество двусторонних А3

        var lastPage = this.numPages - 1; //  номер последней страницы

        var t = app.thermometer;

        t.duration = spreadCount;
        t.begin();

        for (i = 0; i < spreadCount; i++) {

            t.value =  i;
            t.text = "Reordering spread: " + i.toString();
            if (t.cancelled) break;
            
            //  переставляем первую страницу спуска как последнюю книги
            //  вставляем в конец файла ( lastPage ), 
            // c учетом номера спуска (i * 2 - т.к. в начале все время исчезают по 2 страницы в спуске)
            // и отступом 0 для последней страницы, 1 - для предпоследней
            this.movePage(i * 2 + 0, lastPage - i * 2 - 0);
            //  переставляем последнюю (3) страницу спуска как предпоследнюю книги. 
            // 2 - т.к. нумерация изменилась прошлой итерации
            this.movePage(i * 2 + 2, lastPage - i * 2 - 1);

        }
        t.end();

        // app.endPriv();
    }
    
    catch(e) {
        app.alert(e);
    };
    
})


//  Задваивает количество страниц и каждую пополам, оставляя правую или левую стороны
function splitInHalf(fileNameSuffix) {
    app.beginPriv();
    
    try {
        //  сохраним под новым именем
        var newName = this.path;
        var filename = newName.replace(/\.pdf/i, fileNameSuffix); // сохраним копию документа и будем работать с копией

        this.saveAs(filename);

        // сдублируем страницы, чтобы потом сделать из них левые/правые страницы спуска
        var num = this.numPages; //  текущее количество страниц в файле
        var t = app.thermometer;
        t.duration = num;
        t.begin();

        for (var i = 0; i < num; i++) {
            
            t.value =  i;
            t.text = "Duplicate page: " + i.toString();
            
            if (t.cancelled) break;
            
            k = i * 2;
            this.insertPages({ nPage: k, cPath:newName, nStart:i });
        };

        t.end();


        //  сделаем левые/правые страницы
        //  можно воткнуть порезку в предыдущий цикл, где вставляются страницы, но преимущество по скорости минимально, а читаться будет хуже
        var num2 = this.numPages; // текущее количество страниц в файле. Чтобы на каждом проходе не обращаться к this.numPages 
        
        t = app.thermometer;
        t.duration = num2;
        t.begin();

        for (i = 0; i < num2; i += 2) {
                
            t.value =  i;
            t.text = "Define Left/Right Pages: " + i.toString();
            if (t.cancelled) break;

            var aRect = this.getPageBox("Media", i);
            var halfW = (aRect[2] - aRect[0]) / 2;


            //  MediaBox левые страницы
            var mRect = aRect.concat();
            mRect[2] = mRect[0] + halfW;

            //  trimbox
            var tRect = aRect.concat();
            tRect[0] = 0;
            tRect[2] = halfW;
            
            //  CropBox
            var cRect = tRect.concat();
            cRect[2] = cRect[2] - 0.0001; //  нужно, чтобы CropBox был гарантированно меньше, чем TrimBox. особенность спецификации

            this.setPageBoxes("Media", i, i, mRect);
            this.setPageBoxes("Crop",  i, i, cRect);
            this.setPageBoxes("Trim",  i, i, tRect);

            //  MediaBox правые страницы
            mRect = aRect.concat();
            mRect[0] = mRect[2] - halfW;

            this.setPageBoxes("Media", i + 1, i + 1, mRect);
            this.setPageBoxes("Crop",  i + 1, i + 1, cRect);
            this.setPageBoxes("Trim",  i + 1, i + 1, tRect);
        };

        t.end();
    
        app.endPriv();
    }
    
    catch(e) {
        app.alert(e);
    };
    
}

